## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "#>"
)

## ----setup--------------------------------------------------------------------
library(tidyverse)
library(drugfindR)

## ----install------------------------------------------------------------------
#  devtools::install_github("CogDisResLab/drugfindR")

## ----load_signature-----------------------------------------------------------
# Load the signature from the paper

diffexp <- read_tsv("https://zenodo.org/records/10265182/files/dCovid_diffexp.tsv")

# Take a look at the signature

head(diffexp) |>
    knitr::kable()

## ----prepare_signature--------------------------------------------------------
# Prepare the signature for analysis
# The only thing that is different from the defaults is the gene_column
# However, we will specify all three arguments for clarity

signature <- prepare_signature(diffexp, gene_column = "hgnc_symbol", logfc_column = "logFC", pval_column = "PValue")

# Take a look at the signature

head(signature) |>
    knitr::kable()

## ----filter_signature_up------------------------------------------------------
# Filter the signature to only include genes that are upregulated by at least 1.5 logFC

filtered_signature_up <- filter_signature(signature, direction = "up", threshold = 1.5)

filtered_signature_up |>
    head() |>
    knitr::kable()

## ----filter_signature_dn------------------------------------------------------
# Filter the signature to only include genes that are downregulated by at least 1.5 logFC
filtered_signature_dn <- filter_signature(signature, direction = "down", threshold = 1.5)

filtered_signature_dn |>
    head() |>
    knitr::kable()

## ----get_concordants----------------------------------------------------------
# Get the concordant signatures for the upregulated signature

up_concordants <- get_concordants(filtered_signature_up, ilincs_library = "CP", sig_direction = "up")

up_concordants |>
    head() |>
    knitr::kable()

# Get the concordant signatures for the downregulated signature

dn_concordants <- get_concordants(filtered_signature_dn, ilincs_library = "CP", sig_direction = "down")

dn_concordants |>
    head() |>
    knitr::kable()

## ----consensus_concordants----------------------------------------------------
# Get the consensus concordant signatures for the upregulated signature

consensus <- consensus_concordants(up_concordants, dn_concordants, paired = TRUE, cutoff = 0.2)

consensus |>
    head() |>
    knitr::kable()

## ----investigate_signature----------------------------------------------------
investigated <- investigate_signature(diffexp,
    output_lib = "CP", filter_threshold = 1.5,
    gene_column = "hgnc_symbol", logfc_column = "logFC", pval_column = "PValue"
)

investigated |>
    head() |>
    knitr::kable()

## ----sessionInfo--------------------------------------------------------------
devtools::session_info()

